/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: dx3_incident_episodes.sas                                                           |
|     Create incident ICD-9 3-digit diagnosis episodes                                                |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: November 26, 2015                                                                           |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|      - Multiple DX_3_Digit tables split by time segment                                             |
|      - PatKeys table with Partition-ID                                                              |
|      - Bridged Enrollment Spans                                                                     |
|   Program outputs:                                                                                  |
|      - Multiple incident ICD-9 3-digit diagnosis episode tables split by patient partition and by   |
|        lookback period                                                                              |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

/* 11/23/15 - MR: */
/*            - Modify to move creation of DPTemp.Enr_Spans_&P partitions to ENR_ETL program */
/* 12/07/15 -MR */
/*            - Modify name of bridged enrollment spans from DPTemp.Enr_Spans_Bridged.. to DPTemp.Enr_Bridged_Spans.. */
/* 01/21/16 -MR */
/*          - JIRA QCA0-5: Ensure all variables on final SAS datasets used for exporting have the */
/*            correct attributes (e.g. length, format, label) that match the specifications. Towards that goal, we */
/*            reset the length of Code from 5 bytes to 3 bytes in this program so that length will be propagated to */
/*            the final SAS dataset used for exporting the incident diagnosis summary table. */


** TASK SUMMARY ;
** -------------;
** 1- SPLIT DIAGNOSIS CLAIMS INTO PATIENT PARTITIONS;
**    - Split data into partitions to speed up linking and summarizing steps;
** 2- LINK DIAGNOSIS CLAIMS FOR EACH PARTITION;
**    - Link with enrollment spans;
** 3- AGGREGATE SAME-DAY DIAGNOSIS CLAIMS FOR EACH PARTITION;
**    - Dedup same-day claims, while transposing care-setting strata;
** 4- SELECT INCIDENT DIAGNOSIS EVENTS FOR EACH PARTITION;
**    - Select all qualifying candidates for each lookback period strata;

%macro DX3_INCIDENT_EPISODES (DEBUG=);

%put;
%put *** TASK 1- SPLIT DIAGNOSIS CLAIMS AND BRIDGED ENR SPANS INTO PATIENT PARTITIONS;
%put;

  ** Retrieve N_Partitions;
  data _null_;
    set DPTemp.N_Partitions;
    call symputx(Name, Value,'L');
  run;

  ** Split diagnosis claim time-period subsets into patient partition subsets;
  %partition_splits ( InDsn=DPTemp.dx_3_digit: ,
                      OutDsn=DX3_ ,
                      numsets=&N_Partitions ,
                      lkpfile=DPTemp.PatKeyPartition (keep=PatKey Partition) ,
                      PatID=PatKey
                      );


%put ;
%put ************** BEGIN PARTITION LOOP PROCESSING ******************;
%put ;
  %local P;
  %DO P = 1 %to &N_Partitions;  /* BEGIN PARTITION LOOP */
     %put;
     %put  *** FOR PARTITION &P , ;
     %put  *** TASK 2 - LINK THE DIAGNOSIS CLAIMS TO ENROLLMENT;
     %put;

     proc SQL _method;
        CREATE table DX3_ENR_&P as
           SELECT  dx.PatKey, dx.Code length=3, dx.ADate, dx.Period,
                   dx.AgeEventGroupKey, dx.Sex, dx.Setting,
                   enr.EStart , enr.EEnd
             FROM DX3_&P as dx
                      INNER JOIN
                  DPTemp.Enr_Bridged_Spans_&P as enr
             ON dx.PatKey = enr.PatKey and (enr.EStart <= dx.ADate <= enr.EEnd)
             ORDER BY dx.PatKey, dx.Code, dx.ADate
           ;
     quit;

     %if %upcase(&DEBUG) eq N %then %do ;
        %put ** Delete datasets no longer needed;
        proc datasets lib=work nolist nowarn nodetails;
           delete DX3_&P  ;
        quit;
     %end;


     %put;
     %put  *** FOR PARTITION &P , ;
     %put  *** TASK 3 - AGGREGATE SAME-DAY DIAGNOSIS CLAIMS,;
     %put  ***          WHILE TRANSPOSING CARE-SETTING STRATA;
     %put;

     *** Aggregating same-day claims prevents over-counting of events,;
     *** retaining care-setting booleans;
     DATA DX3_UNIQUE_&P;
        SET DX3_ENR_&P;
        BY PatKey Code ADate;

       * Create same-day retained care-setting booleans (0/1);
       length AV ED IP 3.;
       retain AV ED IP .;

       * Initialize bools for each new date;
       if first.ADate then do;
          AV=0; ED=0; IP=0;
       end;

       select ;
         when (setting='AV') AV=1;  /*Ambulatory Visit*/
         when (setting='ED') ED=1;  /*Emergency Department*/
         when (setting='IP') IP=1;  /*Inpatient Admission*/
         otherwise;
       end;

       * Keep last record per date, with care-setting info retained;
       if last.ADate;
       drop setting;
     RUN;

     %if %upcase(&DEBUG) eq N %then %do ;
        %put ** Delete datasets no longer needed;
        proc datasets lib=work nolist nowarn nodetails;
           delete DX3_ENR_&P;
        quit;
     %end;

     %put;
     %put  *** FOR PARTITION &P , ;
     %put  *** TASK 4 - SELECT INCIDENT DIAGNOSIS EVENTS FOR EACH LOOKBACK;
     %put;

     *** Select all qualifying candidates for incident diagnosis events for;
     *** each of the lookback periods;
     DATA DPTemp.DX3_INCIDENT_EPISODE_LB90_&P (sortedby=PatKey Code ADate)
          DPTemp.DX3_INCIDENT_EPISODE_LB180_&P (sortedby=PatKey Code ADate)
          DPTemp.DX3_INCIDENT_EPISODE_LB270_&P (sortedby=PatKey Code ADate) ;
        SET DX3_UNIQUE_&P;
        BY PatKey Code ADate;

        format PriorDate &DateFMT.;
        PriorDate = lag(ADate);
        if first.Code then PriorDate =.;

       ** Output qualifying incident diagnosis events for each lookback;
       length LookBack 3. LookBackDt 4.; format LookBackDt &DateFMT.;
       do LookBack = 90 to 270 by 90;
         LookBackDt = (ADate-LookBack);
         if (EStart <= LookBackDt) and (PriorDate < LookBackDt) then do;
            select (LookBack);
               when ( 90) output DPTemp.DX3_INCIDENT_EPISODE_LB90_&P;
               when (180) output DPTemp.DX3_INCIDENT_EPISODE_LB180_&P;
               when (270) output DPTemp.DX3_INCIDENT_EPISODE_LB270_&P;
               otherwise;
            end;
         end;
         else do;
            leave;  /* First non-incident occurrence breaks lookback loop */
         end;
       end;

        %if %upcase(&DEBUG.) eq N %then %do;
          drop PriorDate LookBackDt EStart EEnd;
        %end;
     RUN;

     %if %upcase(&DEBUG) eq N %then %do ;
        %put ** Delete datasets no longer needed;
        proc datasets lib=work nolist nowarn nodetails;
           delete DX3_UNIQUE_&P ;
        quit;
     %end;


  %END; /* END PARTITION LOOP */
%put ;
%put ************** END PARTITION LOOP PROCESSING ******************;
%put ;

%mend DX3_INCIDENT_EPISODES;
%DX3_INCIDENT_EPISODES (DEBUG=&DEBUG.);

* End of dx3_incident_episodes.sas;
