/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: dx_code_files.sas                                                                   |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: September 24, 2015                                                                          |
|   MOD 10/18/15: Modified use of macro variable "Age_vs_Period"                                      |
|   MOD 10/20/15: Enabled carrying of the SORTEDBY= attribute to output datasets                      |
|   MOD 11/17/15: Set this program to generate files separated by code length only                    |
|   MOD 12/08/15: Use a single DS view as input instead of multiple time-split DS                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       Procedure records processed by DX_ETL                                                         |
|   Program outputs:                                                                                  |
|       Multiple DX_3_Digit temporary tables split by time segment                                    |
|       Multiple DX_4_Digit temporary tables split by time segment                                    |
|       Multiple DX_5_Digit temporary tables split by time segment                                    |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

* MOD: 09/24/15 -MR: 1) Tweak StratVars order for from most levels to least levels
                        StratVars=AgeTimePeriodGroupKey (10 levels) Period (5-13 levels) Sex (2 levels)
                     2) Save intermediate dx_N_digit&Year files needed for subsequent modules to DPTemp
* MOD: 10/14/15 -RR: 1) Set StratVars order dynamically, based on use of global Age_vs_Period parm
* MOD: 10/20/15 -RR: 1) Set SORTEDBY attribute for output datasets;
* MOD: 1/9/16   -RR: 1) Ensure that SORTEDBY variables are found in list of kept variables;

*** Create three new temporary tables as follows:
*** 1 DX_3_Digit: Keep all variables and extract only the left-most 3 characters of DX.
*** 2 DX_4_Digit: Keep all variables and extract only the left-most 4 characters of DX, but only when the 4th character is not blank.
*** 3 DX_5_Digit: Keep all variables and extract only the left-most 5 characters of DX, but only when the 4th and 5th characters are both not blank;

%*** The counts for the rows in the 3 tables should have this relationship: DX_3_Digit >= DX_4_Digit >= DX_5_Digit.;

%macro dx_code_files_years;
  %local Year SortedByDx dx3Keep dx45Keep SortedByDx3 SortedByDx45;

  %*** Get the SORTEDBY attribute from the view, for all output datasets;
  %let SortedByDx=%retrieve_sortedby(dsname=DPTemp.DXV,FullExpr=N);
  %put &=SortedByDx;

  %*** Check that SORTEDBY variable are in the list of kept variables;
  %* Set lists of variables to be kept;

  %let dx3Keep=ADate AgeEventGroupKey AgeTimePeriodGroupKey Code PatKey Period Setting Sex;
  %let dx45Keep=AgeTimePeriodGroupKey Code PatKey Period Setting Sex;
  %* Filter SortedBy vars for dx3;
  %SortedBy_check(
    SourceWrds=&SortedbyDx.,
    ReturnVar=SortedByDx3,
    CheckWrds=&dx3Keep.
  );
  %** If there are SortedBy variables, then create full data set option expression;
  %if %length(&SortedByDx3.)>0 %then %let SortedByDx3=%str(SortedBy=&SortedByDx3.);
  %* Filter SortedBy vars for dx4 and dx5;
  %SortedBy_check(
    SourceWrds=&SortedbyDx.,
    ReturnVar=SortedByDx45,
    CheckWrds=&dx45Keep.
  );
  %** If there are SortedBy variables, then create full data set option expression;
  %if %length(&SortedByDx45.)>0 %then %let SortedByDx45=%str(SortedBy=&SortedByDx45.);

  data
    %do Year=&DP_StartYear. %to &DP_EndYear.;
      DPTemp.dx_3_digit&Year.(Keep=&dx3Keep. &SortedByDx3.)
      DPTemp.dx_4_digit&Year.(Keep=&dx45Keep. &SortedByDx45.)
      DPTemp.dx_5_digit&Year.(Keep=&dx45Keep. &SortedByDx45.)
    %end;
    ;
    set DPTEMP.DXV(rename=(Code=_Code));
    %do Year=&DP_StartYear. %to &DP_EndYear.;
      if Period eq "&Year." then do;
        ** 3-digit always gets written out;
        Code=substr(_Code,1,3);
        output DPTemp.dx_3_digit&Year.;
        ** 4-digit;
        if substr(_Code,4,1) NE " " then do;
          Code=substr(_Code,1,4);
          output DPTemp.dx_4_digit&Year.;
        end;
        else return;  * 5-digit would not be satisfied either, if 4th position is blank;
        ** 5-digit;
        if substr(_Code,5,1) NE " " then do;
          Code=substr(_Code,1,5);
          output DPTemp.dx_5_digit&Year.;
        end;
        return;  * Done with year and checking Dx length;
      end;
    %end;
    drop _: ;
  run;

  %*** QC on counts of files;
  %if &DEBUG.=Y %then %do;
    *** DEBUGGING: Check counts of 3 datasets;
    %do Year=&DP_StartYear. %to &DP_EndYear.;
      %put %CMPRES(Counts of 3 &Year. Dx files are: Dx3 = %dsn_obs(dsn=DPTemp.dx_3_digit&Year.) >=
        Dx4 = %dsn_obs(dsn=DPTemp.dx_4_digit&Year.) >= Dx5 = %dsn_obs(dsn=DPTemp.dx_5_digit&Year.));
    %end;
  %end;
%mend;
%dx_code_files_years;

* End of dx_code_files.sas;
