/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: incident_dx3.sas                                                                    |
|     Create incident ICD-9 3-digit summary table                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: November 26, 2015                                                                           |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|      - Multiple incident ICD-9 3-digit diagnosis episode tables split by patient partition and by   |
|        lookback period                                                                              |
|      - Dx_icd9_3dig_lookup table                                                                    |
|   Program outputs:                                                                                  |
|      - Incident ICD-9 3-digit summary table                                                         |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

/* 01/26/16 -MR */
/*            - JIRA QCA0-5: Ensure all variables on final SAS datasets used for exporting have the */
/*              correct attributes (e.g. length, format, label) that match the specifications       */

%macro INCIDENT_DX3 (DEBUG=N) ;

%put ;
%put ************** BEGIN PARTITION-LOOKBACK LOOP PROCESSING ******************;
%put ;

  ** Retrieve N_Partitions metadata;
  data _null_;
    set DPTemp.N_Partitions;
    call symputx(Name, Value,'L');
  run;

  %local P;
  %DO P = 1 %to &N_Partitions;  /* BEGIN PARTITION LOOP */

     %local L;
     %DO L = 90 %to 270 %by 90;  /* BEGIN LOOKBACK LOOP */

        %put;
        %put  *** FOR PARTITION &P AND LOOKBACK &L:;
        %put  *** TASK 1: AGGREGATE TO PATIENT-CODE-STRATIFICATION LEVEL;
        %put;

        proc SORT
            DATA=DPTemp.DX3_Incident_Episode_LB&L._&P
                ( keep=PatKey Code Period AgeEventGroupKey Sex AV ED IP )
            OUT=DX3_Incident_Episode_LB&L._&P;
            BY PatKey Code Period AgeEventGroupKey Sex;
        run;

        DATA DX3_Pat_Strata_LB&L._&P ;
          SET DX3_Incident_Episode_LB&L._&P (rename=(AV=_AV ED=_ED IP=_IP));
          BY PatKey Code Period AgeEventGroupKey Sex;

          * Create retained care-setting event counts;
          length AN AV ED IP 3.;
          retain AN AV ED IP .;

          * Create retained care-setting patient bools (0/1);
          length AN_PATS AV_PATS ED_PATS IP_PATS 3.;
          retain AN_PATS AV_PATS ED_PATS IP_PATS .;

          * Initialize care-setting event counts and care-setting patient bools;
          * for each stratification;
          if first.Sex then do;
             AN=0; AV=0; ED=0; IP=0;
             AN_PATS=0; AV_PATS=0; ED_PATS=0; IP_PATS=0;
          end;

          * Increment care-setting event counts;
          AN + 1;
          AV + _AV;
          ED + _ED;
          IP + _IP;

          * Set the care-setting patient bools for last record in each strata,;
          * then output the summarized patient-code-strata record;
          if last.Sex then do;
             AN_PATS = (AN > 0);
             AV_PATS = (AV > 0);
             ED_PATS = (ED > 0);
             IP_PATS = (IP > 0);
             length Setting $2. Members 3. Events 3.;
             if AN_PATS eq 1 then
                do;
                  Setting='AN'; Members=AN_PATS; Events=AN; output;
                end;
             if AV_PATS eq 1 then
                do;
                  Setting='AV'; Members=AV_PATS; Events=AV; output;
                end;
             if ED_PATS eq 1 then
                do;
                  Setting='ED'; Members=ED_PATS; Events=ED; output;
                end;
             if IP_PATS eq 1 then
                do;
                  Setting='IP'; Members=IP_PATS; Events=IP; output;
                end;
           end;
          drop _: ;
          %if %upcase(&DEBUG.) ne Y %then %do;
             drop AN: AV: ED: IP: ;
          %end;
        RUN;

        %put;
        %put  *** FOR PARTITION &P AND LOOKBACK &L:;
        %put  *** TASK 2: AGGREGATE TO CODE-STRATIFICATION LEVEL;
        %put;

        %** DEV Note: order group-by vars from most to least number of distinct values for best performance;
        proc SQL _method ;
          CREATE table DX3_Strata_LB&L._&P as
            SELECT AgeEventGroupKey ,
                   Sex ,
                   Period ,
                   Code ,
                   Setting ,
                   SUM(Members) as Members&L ,
                   SUM(Events) as Events&L
               FROM DX3_Pat_Strata_LB&L._&P
               GROUP BY Code, Period, AgeEventGroupKey, Setting, Sex
            ;
         quit;


     %END; /* END LOOKBACK LOOP */

  %END; /* END PARTITION LOOP */

%put ;
%put ************** END PARTITION-LOOKBACK LOOP PROCESSING ******************;
%put ;

%put;
%put  *** TASK 3: CONCATENATE PARTITION-LOOKBACK AGGREGATE SUBSETS INTO ;
%put  ***         LOOKBACK SUBSETS AND RE-AGGREGATE BY SUMMARY KEYS;
%put;

  %** Define helper macro for concatenating and aggregating;
  %macro _CONCAT_SUMM_ ( InDSN=, OutDSN=);
    %** Create temporary view of subsets;
    DATA Temp_view /VIEW=Temp_View ;
      SET &InDSN. ;
    RUN;

    %** Aggregate view by summary keys;
    %** DEV Note: order group-by vars from most to least # of distinct values for performance;
    proc SQL _method ;
      CREATE table &OutDSN. as
          SELECT AgeEventGroupKey ,
                 Sex ,
                 Period ,
                 Code ,
                 Setting ,
                 SUM(Members&L.) as Members&L ,
                 SUM(Events&L.) as Events&L
           FROM Temp_View
           GROUP BY Code, Period, AgeEventGroupKey, Setting, Sex
       ;
     quit;

    %** Delete temporary view;
     proc datasets lib=WORK nolist nodetails nowarn;
        delete Temp_View ;
     quit ;
  %mend;

  %local InDSN OutDSN L;
  %let InDSN=DX3_Strata_LB;
  %let OutDSN=Summ_DX3_Strata_LB;

  %** Call helper macro for concatenating/aggregating subsets:;
  %do L = 90 %to 270 %by 90;

    %put;
    %put ** LOOKBACK PERIOD &L, CONCATENATE ALL OF THE PARTITIONS;
    %put ** AND RE-AGGREGATE THE RESULTS BY THE SUMMARY KEYS;
    %put;

     %_CONCAT_SUMM_ ( InDSN=%str(&InDSN.&L.:),
                      OutDSN= &OutDSN.&L.
                    )
  %end;

%put;
%put  *** TASK 4: MERGE ALL AGGREGATED LOOKBACK SUBSETS TOGETHER BY THE SUMMARY KEYS;
%put;

  DATA Summ_DX3_Strata_ALL_LB;
    MERGE Summ_DX3_Strata_LB90
          Summ_DX3_Strata_LB180
          Summ_DX3_Strata_LB270;
    BY Code Period AgeEventGroupKey Setting Sex;

    ** Replace numeric missing values with 0;
    array M(*) _NUMERIC_;
    do i=1 to dim(M);
       if M(i) eq . then M(i)=0;
    end;

    drop i;
  RUN;
  %put Note: We expect LB90 Obs count > LB180 Obs count > LB270 Obs count;

%put;
%put  *** TASK 5: LINK IN DESCRIPTIONS AND FINALIZE;
%put;

  %put *** Link in Code Description;
  %put;


  proc SQL _method;
     CREATE table Summ_DX3_Final as
       SELECT dx.*, lk.srt_descrip as DXName length=35
         FROM Summ_DX3_Strata_ALL_LB as dx
                  INNER JOIN
              InFolder.Dx_icd9_3dig_lookup as lk
         ON dx.Code = lk.Code
     ;
  quit;

  %put *** Link in AgeGroup description and AgeGroupID;
  %put;

  DATA Summ_DX3_Final;
    SET Summ_DX3_Final;
      Age_Group = put(AgeEventGroupKey,$AgeGroupID_to_Value.);
      length Age_Group_ID 3.;
      Age_Group_ID = input(put(AgeEventGroupKey,$AgeGroupID_to_True_ID.),2.);
      drop AgeEventGroupKey;
  RUN;

  %put *** Finishing touches to create final SAS dataset formatted exactly as required;
  %put;

  proc SORT DATA=Summ_DX3_Final;
     BY Age_Group_ID Sex Period Code Setting;
  run;

  %local keys measures;
  %let keys = Age_Group Sex Period Code DXName Setting;
  %let measures = Members90 Events90 Members180 Events180 Members270 Events270;

  /* Note: Use attribute-statements "Jiu-Jitsu" in order to: */
  /*   (a) force variable order (a nice side-effect of when placed between DATA and SET statements */
  /*   (b) set default numeric format length                     */
  /*   (c) override format length for numeric variable Age_ID */
  /*   (d) use default format length for character vars which default to physical var length */

  /* Dev/QC Note: Character formats are now explicitly set to comply with specifications. */
  /*              Likewise, variable labels are now assigned to comply with specifications. */

  DATA DPLocal.Incident_ICD9_Diagnosis;
    informat &keys. &measures. Age_Group_ID;
    SET Summ_DX3_Final ;

    format Age_Group $5. Code $3. DxName $35. Period $4. Setting $2. Sex $1. ;
    format _NUMERIC_ 15. ;
    format Age_Group_ID 2. ;
    informat _ALL_ ;

    label Age_Group     = 'Age Group'
          Sex           = 'Sex'
          Period        = 'Year'
          Code          = '3-digit Dx Code'
          DxName        = 'Diagnosis Name'
          Setting       = 'Setting'
          Members90     = 'Members in 90-day lookback'
          Events90      = '# of Events from 90-day lookback'
          Members180    = 'Members in 180-day lookback'
          Events180     = '# of Events from 180-day lookback'
          Members270    = 'Members in 270-day lookback'
          Events270     = '# of Events from 270-day lookback'
          Age_Group_ID  = 'ID' ;

  RUN;

%mend INCIDENT_DX3 ;

%INCIDENT_DX3 (DEBUG=&DEBUG.);

* End of incident_dx3.sas;
