/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: qt_utility_macros.sas                                                               |
|   A place for defining summary tables specific macros that may be used across modules               |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: August 27, 2015                                                                             |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       NONE                                                                                          |
|                                                                                                     |
|   Program outputs:                                                                                  |
|       compiled macros                                                                               |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/


%**************** BEGIN LIST OF MACROS INCLUDED ***********************************************;
%*;
%*  SetUp               -- Always include first, performs dynamic setup tasks;
%*  Access_Export_mdb   -- Export SAS table one of several MS ACCESS database formats;
%*  Age_vs_Period       -- Lists age and period variables in order of most to least levels;
%*  ExportFiles         -- Export final results to text and/or MS Access formats;
%*  Partition_Patients  -- Creates a patient with partition-id variable;
%*  Partition_Splits    -- Splits a dataset into partitions based on partition-id;
%*  SigFiles_Finalize   -- Finalizes run-times signature file and creates overall signature file;
%*  SortedBy_Check      -- Validates that variables listed on SORTEDBY are on output dataset;
%*  Text_Export         -- Export SAS table to text format;
%*  Transpose           -- Transpose a single measure;
%*  UpDateSigFile       -- Updates signature file with module name, start/stop times and duration;
%*  ValidateParm        -- Validates parm against set of valid values, aborts if value not valid;
%*;
%**************** END LIST OF MACROS INCLUDED *************************************************;

%*** Macro for performing dynamic setup tasks for this package -- must include first;
%MACRO SETUP ;
  /* Assign DPtemp path location and SAS libref */
  %if %length(&DPTEMP)=0 %then %do ;
     %let DPTEMP = &DPLOCAL.dptemp ;
     options DLCREATEDIR ;
     libname DPTEMP "&DPTEMP" ;
     options NODLCREATEDIR;
  %end ;
  %else %do ;
     %let DPTEMP = %ms_clean_path(&DPtemp) ;
     %ms_lib(DPTEMP, &DPTEMP)
  %end ;
%mend SETUP ;
%SETUP

%*** Macro to generate Access databases;
%macro Access_Export_mdb(Ds=);
  %local AccessFileName Extension;
  %** Set name of Access file;
  %let AccessFileName=Mini-Sentinel_Summary_Tables;
  %if &AccessFiles.=M %then %let Extension=MDB;
  %else                     %let Extension=ACCDB;

  %if %upcase(&ACCESS_DBMS) eq ACCESSCS %then %do;
     PROC EXPORT DATA= DPLocal.&Ds.
       DBMS=ACCESSCS REPLACE
       OUTTABLE = "&Ds.";
       DATABASE = "&ExportPath.&AccessFileName..&Extension.";
     RUN;
  %end;
  %else %if %upcase(&ACCESS_DBMS) eq ACCESS %then %do;
     PROC EXPORT DATA= DPLocal.&Ds.
       DBMS=ACCESS REPLACE
       OUTTABLE = "&Ds.";
       DATABASE = "&ExportPath.&AccessFileName..&Extension.";
     RUN;
  %end;
%mend Access_Export_mdb;

/*---------------------------------------------------------------------------------------------------*\
|   MACRO NAME: age_vs_period.sas                                                                     |
|     Determines which stratifier has more values: Period (i.e., year) vs. Age groups                 |
|     This is used in aggregation programs                                                            |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: October 14, 2015                                                                            |
|   MOD 10/18/2015: Changed output to be either "AgeTimePeriodGroupKey Period" or                     |
|                                               "Period AgeTimePeriodGroupKey"                        |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       Global macro variable DP_NumYears                                                             |
|   Program outputs:                                                                                  |
|       Global macro variable Age_vs_Period with values as follows:                                   |
|          "AgeTimePeriodGroupKey Period"=Age group has more values than Period                       |
|          "Period AgeTimePeriodGroupKey"=Period has more values than Age group                       |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

%MACRO Age_vs_Period();
  %*** Declare local macro vars;
  %local AgeGroups;
  %*** Declare global macro vars;
  %global Age_vs_Period;
  %*** Hard code number of age groups.  This should stay stable.;
  %let AgeGroups=10;
  %*** Compare # of age groups vs. # of Periods;
  %if %eval(AgeGroups > &DP_NumYears.) %then %do;
    %let Age_vs_Period=%str(AgeTimePeriodGroupKey Period);
  %end;
  %else %do;
    %let Age_vs_Period=%str(Period AgeTimePeriodGroupKey);
  %end;
  %put &=Age_vs_Period.;
%MEND;

%**** Macro to control exporting tables;
%macro ExportFiles;

  %*** Retrieve export macro variables;
  data _null_;
    set DPtemp.ExportFilesMeta;
    call symputx(name, value, 'L');
  run ;

  %*** Name local variables;
  %local ExportTables d tbl;
  %** List the tables to be exported;
  %let ExportTables=Age_Groups
                    Enrollment
                    ICD9_Diagnosis
                    ICD9_Diagnosis_4_Digit
                    ICD9_Diagnosis_5_Digit
                    HCPCS ICD9_Procedure
                    ICD9_Procedure_4_Digit
                    Drug_Class Generic_Name
                    Incident_ICD9_Diagnosis
                    Incident_Drug_Class
                    Incident_Generic_Name;
  %let ExportTables = %cmpres(&ExportTables);

  %*** If text files are requested;
  %if &TextFiles. NE N %then %do;
    %********************************************* Text Files ********************************************* ;
    %*** Loop through all tables, check that they exist in DPLocal, and then export;
    %do d=1 %to %sysfunc(countw(&ExportTables.));
      %* Get a table name;
      %let tbl=%scan(&ExportTables.,&d.,%str( ));
      %if %DSN_EXISTS(DPLocal.&tbl.)=1 %then %do;
        %Text_Export(ds=&tbl.);
      %end;
      %else %do;
        %** Print message to log;
        %put;
        %put %upcase(war)%upcase(ning:) Table &tbl. does not exist and will not be exported to a text file.;
        %put %upcase(war)%upcase(ning:) This issue should be investigated by the DP and SOC.;
        %put;
      %end;
    %end;
  %end;

  %*** If Access databases are requested;
  %if &ProcessAccess.=Y %then %do;
    %********************************************* Access Databases ********************************************* ;
    %*** Loop through all tables, check that they exist in DPLocal, and then export;
    %do d=1 %to %sysfunc(countw(&ExportTables.));
      %* Get a table name;
      %let tbl=%scan(&ExportTables.,&d.,%str( ));
      %if %DSN_EXISTS(DPLocal.&tbl.)=1 %then %do;
        %Access_Export_mdb(Ds=&tbl.);
      %end;
      %else %do;
        %** Print message to log;
        %put;
        %put %upcase(war)%upcase(ning:) Table &tbl. does not exist and will not be exported to Access database.;
        %put %upcase(war)%upcase(ning:) This issue should be investigated by the DP and SOC.;
        %put;
      %end;
    %end;
  %end;

%MEND;

%MACRO Partition_Patients(InDsn=, OutDsn=, N_Partitions=, PatID=, OtherKeepVars=);
%*** Macro creates randomly assigned partition-id for each patient;

  %if %length(&PatID.) eq 0 %then %do;
     %let PatID=PatID;
  %end;
  %local Select_OtherKeepVars;
  %if %length(&OtherKeepVars.) > 0 %then %do;
      %let Select_OtherKeepVars = %insertchar(&OtherKeepVars.);
      %let Select_OtherKeepVars =&Select_OtherKeepVars. ,;
  %end;

  proc sort nodupkey data=&InDsn (keep=&PatID. &OtherKeepVars.) out=unique_pats;
    by &PatID. ;
  run ;

  proc sql _method ;
    create table pats as
      select &PatID., &Select_OtherKeepVars. ranuni(1776) as rn
      from unique_pats
      order by calculated rn
    ;
  quit ;

  data pats ;
    set pats ;
    by rn ;
    pat+1 ;
    drop rn ;
  run ;

  proc rank group=&N_Partitions data=pats out=pats (drop=pat) ;
    var pat ;
    ranks group ;
  run ;

  data &outDsn. ;
    set pats ;
    length partition 3. ;
    partition = group + 1 ;   /* first group starts at 0 which is not very intuitive */
    drop group ;
  run ;
%MEND Partition_Patients;


%MACRO Partition_Splits (InDsn=, OutDsn=, tbl_suffix=, numsets=, lkpfile=, PatID=, SortBy=);
%*** Macro splits input table into X number of partitioned tables;

  %if %length(&PatID) eq 0 %then %do;
     %let PatID=PatID;
  %end;

 %* Dynamically build the output datasets based on the number of partitions.;
 %local dssplit i numsets;
 %let dssplit=;
 %do i=1 %to &numsets.;
   %let dssplit=&dssplit. %str(&OutDsn.&tbl_suffix.&i. (drop=partition));
 %end;

 %* Dynamically build the if condition to output to the proper partition.;
 %local whichsplit;
 %do i=1 %to &numsets.;
   %if &i eq 1 %then %do;
     %let whichsplit=%str(if partition = &i. then output &OutDsn.&tbl_suffix.&i.;);
   %end;
   %else %do;
     %let whichsplit=&whichsplit. %str(else if partition = &i. then output &OutDsn.&tbl_suffix.&i.;);
   %end;
 %end;

 *** Hash dataset to lookup patid and output to partition. ;
 data &dssplit.;
   if _n_ = 0 then do;
     set &lkpfile.;
   end;

   if _n_ = 1 then do;
     declare hash split(DATASET:"&lkpfile.");
     split.defineKey("&PatID.");
     split.defineData('partition');
     split.defineDone();
   end;

   set &InDsn.;
   if split.find()=0;
   &whichsplit.;
 run;

  %if %length(&SortBy) > 0 %then %do;
     %do i=1 %to &numsets.;
       %put;
       %put *** FOR PARTITION &I.:;
       %put *** Sort dataset &OutDsn.&tbl_suffix.&i. by &SortBy.;
       %put;
       proc sort data=&OutDsn.&tbl_suffix.&i. out=&OutDsn.&tbl_suffix.&i. ;
         by &SortBy. ;
       run ;
     %end;
  %end;


%MEND Partition_Splits;


%*** Macro to cleanup module runtime signature file and create overall signature file ;
%MACRO SigFiles_Finalize(indsn, outdsn) ;

/* 10/08/15 - MR */
/*            - Fix bug impacting Totals */
/* 02/10/16 -MR  */
/*            - Only create final signature file if &INDS exists and has obs */

 %local CheckOk;
 %let CheckOk=1;
 %if %dsn_exists(&INDSN) eq 0 %then %do;
   %let CheckOk=0;
   %put ;
   %put %upcase(w)%upcase(arning:) &INDSN does not exist, therefore we cannot create &OUTDSN;
   %put ;   
 %end;
 %else %do;
   %if %dsn_obs(&INDSN) eq 0 %then %do;
     %let CheckOk=0;  
     %put ;
     %put %upcase(w)%upcase(arning:) &INDSN has 0 obs, therefore we cannot create &OUTDSN;
     %put ;     
   %end;
 %end;

 %if &CheckOk eq 1 %then %do;  /* BEGIN CheckOK Block */
  ** Clean up module runtime signature file by keeping only most recent runtime per module ;
  proc sort data=&INDSN out=_ModuleRuntimes ;
    by ProcessName StartTime ;
  run ;

  data _ModuleRuntimes ;
    set _ModuleRuntimes ;
    where index(upcase(ProcessName), 'TOTAL') = 0 ;
    by ProcessName StartTime ;
    if last.ProcessName ;
    keep ProcessName ProcessRC StartTime EndTime RunTime
         RunTimeDays RunTimeHours RunTimeMinutes RunTimeSeconds ;
  run ;

  proc sql ;
     create table _TotalRuntime as
        select '** TOTAL RUNTIME **' as ProcessName length=50 ,
               min(StartTime) as StartTime format=DATETIME19. ,
               max(EndTime) as EndTime format=DATETIME19. ,
               sum(RunTime) as RunTime format=TIME12. ,
               sum(RunTimeDays) as RunTimeDays format=12.2 ,
               sum(RunTimeHours) as RunTimeHours format=12.2 ,
               sum(RunTimeMinutes) as RunTimeMinutes format=12.2 ,
               sum(RunTimeSeconds) as RunTimeSeconds format=12.2
         from _ModuleRuntimes ;
   quit ;

  ** Sort ModuleRunTimes by run-sequence order;
  proc sort data=_ModuleRuntimes out=_Run_Order;
    by StartTime EndTime;
  run;

  ** Create run-sequence order variable;
  data _Run_Order;
    set _Run_Order;
    by StartTime EndTime;
    length SequenceOrder 3.;
    SequenceOrder+1;
  run;

  ** Append total runtime to module runtimes ds ;
  data &INDSN  ;
     set _Run_Order _TotalRuntime (in=inTot) ;
     if inTot then SequenceOrder = 0;
  run ;

  ** Sort by decending run-time order ;
  proc sort data=&INDSN ;
    by descending RunTime;
  run ;

  ** Calc percent of total run-time for each module;
  data &INDSN;
      informat SequenceOrder ProcessName ProcessRC StartTime EndTime DurationOrder PctTotTime RunTime
               RunTimeDays RunTimeHours RunTimeMinutes RunTimeSeconds ;
    set &INDSN;
    by descending RunTime ;
    retain TotRunTime .; length DurationOrder 3.;
    if SequenceOrder eq 0 then do;
       DurationOrder=0;
       TotRunTime=RunTime;
    end;
    else do;
       DurationOrder+1;
       PctTotTime=(RunTime/TotRunTime);
    end;
    format PctTotTime percent6.2 ;
    drop TotRunTime;
  run;

  %* Format metadata related SAS Options *;
  %local threads memsize realmemsize sortsize maxmemquery;
  %local sumsize bufno bufsize ubufno ubufsize vbufsize compress;
  %let threads = %sysfunc(getoption(threads));
  %let memsize = %sysfunc(inputn(%sysfunc(getoption(memsize)),30.),comma30.) bytes;
  %let realmemsize = %sysfunc(inputn(%sysfunc(getoption(realmemsize)),30.),comma30.) bytes;
  %let sortsize = %sysfunc(inputn(%sysfunc(getoption(sortsize)),30.),comma30.) bytes;
  %let maxmemquery = %sysfunc(inputn(%sysfunc(getoption(maxmemquery)),30.),comma30.) bytes;
  %let sumsize = %sysfunc(inputn(%sysfunc(getoption(sumsize)),30.),comma30.) bytes;
  %let bufno = %sysfunc(inputn(%sysfunc(getoption(bufno)),30.),comma30.);
  %let bufsize = %sysfunc(inputn(%sysfunc(getoption(bufsize)),30.),comma30.) bytes;
  %let ubufno = %sysfunc(inputn(%sysfunc(getoption(ubufno)),30.),comma30.);
  %let ubufsize = %sysfunc(inputn(%sysfunc(getoption(ubufsize)),30.),comma30.) bytes;
  %let vbufsize = %sysfunc(inputn(%sysfunc(getoption(vbufsize)),30.),comma30.) bytes;
  %let compress = %sysfunc(getoption(compress));

  ** Retrieve metadata related to the size of CDM tables in terms of gigabytes ; 
  %local ST_MaxTableSizeGB ST_CDMSizeGB MaxTableSizeGB CDMSizeGB;
  proc sql noprint;
   select max(filesize)/1000000000, sum(filesize)/1000000000 
              into 
          :ST_MaxTableSizeGB, :ST_CDMSizeGB 
      from dictionary.tables
      where libname eq 'INDATA' and memname in (%QuoteLst(&ST_TableList));
   select max(filesize)/1000000000, sum(filesize)/1000000000 
                into 
          :MaxTableSizeGB, :CDMSizeGB 
      from dictionary.tables
      where libname eq 'INDATA' ;      
  quit;  

  ** Create overall signature file ; 
  data _overall_signature ;
    DPID = "&DPID" ;
    SITEID = "&SITEID" ;
    ETL = "&ETL" ;
    DP_MinDate  = put(&DP_MinDate, date9.) ;
    DP_MaxDate  = put(&DP_MaxDate, date9.) ;
    DP_Years = put(&DP_NumYears, 4.) ;
    PackageName = "&PackageName";
    PackageVersion = "&PackageVersion" ;
    MSReqID = "&MSReqID" ;
    MSProjID = "&MSProjID" ;
    MSWPType = "&MSWPType" ;
    MSWPID = "&MSWPID" ;
    MSDPID = "&MSDPID" ;
    MSVerID = "&MSVerID" ;
    set _TotalRuntime (keep=StartTime EndTime RunTime rename=(StartTime=ST EndTime=ET RunTime=RT)) ;
    StartTime = put(ST, DATETIME19.) ;
    EndTime = put(ET, DATETIME19.) ;
    RunTime = put(RT, TIME12.) ;
    drop ST ET RT ;
    OSAbbrevation = "&sysscp.";
    OSName = "&sysscpl.";
    SASVersion = "&sysver.";
    SASVersionLong = "&sysvlong.";
    SASRunType = "&sysenv.";
    CPUCount = "&sysncpu.";
    Threads = "&threads." ;
    MemSize = "&memsize." ;
    RealMemSize = "&realmemsize." ;
    SortSize = "&sortsize." ;
    MaxMemQuery = "&maxmemquery." ;
    SumSize = "&sumsize." ;
    BufNo = "&bufno." ;
    BufSize = "&bufsize." ;
    UBufNo = "&ubufno." ;
    UBufSize = "&ubufsize." ;
    VBufSize = "&vbufsize." ;
    Compress = "&compress.";    
    ST_MaxTableSizeGB = trim("&ST_MaxTableSizeGB");
    ST_CDMSizeGB = trim("&ST_CDMSizeGB");
    MaxTableSizeGB = trim("&MaxTableSizeGB");
    CDMSizeGB = trim("&CDMSizeGB");
    output ; stop ;    
  run ;

  proc transpose data=_overall_signature out=&outdsn (rename=(_NAME_=VAR COL1=VALUE));
    var _ALL_ ;
  run ;

  data &outdsn;
    set &outdsn;
    value=strip(value);
    if value eq "0 bytes" then value = "0 (no-limit)";
    else if upcase(var) in: ("BUFNO" "UBUFNO") and value eq "0" then value = "0 (no-limit)";
  run;

  proc datasets lib=work nodetails nolist nowarn ;
    delete _Run_Order _overall_signature _ModuleRuntimes _TotalRuntime ;
  quit ;
  
 %end; /* END CheckOK Block */

%MEND SigFiles_Finalize ;


%** sortedby_check.sas;
%*** This macro checks if each of a list of words (e.g., variables) in one list exists in another list.;
%*** Checking is from left to right and any failure (i.e. word/variable not found) stops the process.;

%MACRO sortedby_check(
  SourceWrds=,  /* The list of words being checked                                                                 */
  CheckWrds=,   /* The list of words against which SourceWrds is checked                                           */
  ReturnVar=    /* The name of the macro variable to contain the word list after checking                          */
);

  %local Returned NF wct LoopStop wd;
  %*** Check parms 1 and 2 to ensure that they are filled, return message if not;
  %if %length(&SourceWrds.) = 0 or %length(&CheckWrds.) = 0 %then %do;
    %put;
    %put ************************* SortedBy WARNING **********************;
    %put One or both of input parms SourceWrds and CheckWrds are empty;
    %put Returning null value to variable &ReturnVar.;
    %put *****************************************************************;
    %put;
    %let &ReturnVar.=;
  %end;
  %else %do;
    %*** Display initial source list of words;
    %put Initial &=SourceWrds;
    %*** Force input parms to upper-case;
    %let SourceWrds=%upcase(&SourceWrds.);
    %let CheckWrds=%upcase(&CheckWrds.);
    %*** Initialize local returned parm;
    %let Returned=;
    %*** Initialize local NOT FOUND switch to zero;
    %let NF=0;
    %*** Loop through each word in the SourceWrds value, from right to left;
    %* Set stop value for loop;
    %let LoopStop=%sysfunc(countw(&SourceWrds.,%str( )));
    %do wct=1 %to &LoopStop.;
      %*** Get one word;
      %let wd=%scan(&SourceWrds.,&wct.,%str( ));
      %*** Test if word is NOT in the CheckWrds list;
      %if %sysfunc(indexw(&CheckWrds.,&wd.)) = 0 %then %do;
        %*** Set local switch to True;
        %let NF=1;
        %*** Force leaving the loop;
        %let wct=&LoopStop.;
      %end;
      %else %do;
        %if &NF.=0 %then %do;
          %*** As the word is found ensure that it is returned;
          %let Returned=&Returned. &wd.;
        %end;
      %end;
    %end;
    %*** Set final list of words;
    %let &ReturnVar.=&Returned.;
    %*** Display final source list of words;
  %end;
  %put Final &=ReturnVar &=Returned.;
  %put ;
%MEND;


%*** Compile macro to generate text files;
%MACRO Text_Export(Ds=);
  %local ext;
  %let ext = txt;
  /*
  %if &TextFiles.=T %then %do;
    %let ext=tsv;
  %end;
  %else %if &TextFiles.=C %then %do;
    %let ext=csv;
  %end;
  %else %do;
    %let ext=txt;
  %end;
  */
  PROC EXPORT DATA= dplocal.&Ds.
    OUTFile = "&ExportPath.&ds..&ext."
    replace
    DBMS=dlm;
    %*** Additional control for TAB output;
    %if &TextFiles.=T %then %do;
      delimiter="09"x;
    %end;
    %else %do;
      delimiter="%sysfunc(translate(&TextFiles.,%str(,|),CP))";
    %end;
  RUN;
%MEND;

%*** Define helper macro to transpose a single measure;
%MACRO TRANSPOSE( InDsn=,
                  OutDsn=,
                  Delim=,
                  Prefix=,
                  By=,
                  Id=,
                  Var=);
   %local DelimExpress;
   %if %length(&Delim) > 0 %then %do;
       %let DelimExpress = Delim=&Delim;
   %end;
   PROC TRANSPOSE data=&InDsn.
                  out=&OutDsn. (drop=_name_)
                  &DelimExpress
                  prefix=&Prefix.;
     BY &By.;
     ID &Id.;
     VAR &Var.;
   RUN;
%MEND;


*** Create time utilized dataset with one record for placement into signature file;
%MACRO UpDateSigFile(
  ProcessName=,  /* The name of the process  (50 chars)                   */
  ProcessRC=,    /* The return code from executing the process (optional) */
  StartTime=,    /* The start time for a process as SAS date/time value   */
  EndTime=,      /* The end time for a process as SAS date/time value     */
  OutSigName=    /* Name of output signature dataset                      */
  );
  /* MOD 09/18/15 -MR: Save RunTime as numeric and create derivative */
  /*                  RunTime vars for Days, Hours, Minutes, Seconds */
  /* MOD 11/10/15 -MR: Modify Days, Hours, and Minuetes calculations */
  /* MOD 01/01/12 -MR: Add error handling to prevent creating corrupt DS */
  
  /* Check for corrupt &OuSigName and delete if found */
  %local levels lib dsn;
  %if %dsn_exists(&OutSigName)=1 %then %do;
    %if %dsn_obs(&OutSigName)=0 %then %do;
      %put;
      %put &OutSigName has 0 obs and will be deleted before attempting to recreate;
      %put;
      %let levels = %sysfunc(countw(&OutSigName));
      %if &levels eq 1 %then %do;
        %let lib=WORK;
        %let dsn=&OutSigName;
      %end;
      %else %do;
        %let lib=%scan(&OutSigName,1);
        %let dsn=%scan(&OutSigName,2);
      %end;
      proc datasets lib=&lib nolist nowarn nodetails;
        delete &dsn;
      quit;
    %end;
  %end;
  
  ** Assemble single record;
  data _TimeRecord;
    length ProcessName $50 ProcessRC 4.;
    retain ProcessName "&ProcessName." ProcessRC &ProcessRC. ;
    format StartTime EndTime DATETIME19.
           RunTime TIME12.
           RunTimeDays RunTimeHours
           RunTimeMinutes RunTimeSeconds 12.2 ;
    *** Create Start and Stop variables;
    StartTime=&StartTime.;
    EndTime=  &EndTime.;
    *** Calculate RunTime and format as HH:MM:SS;
    RunTime=(&EndTime. - &StartTime) ;
    *** Breakout Runtime HH:MM:SS into separate RunTime derviative fields ;
    RunTimeSeconds= INTCK('DTSECOND',StartTime,EndTime) ;
    RunTimeMinutes=  RunTimeSeconds/60;
    RunTimeHours= RunTimeMinutes/60;
    RunTimeDays= RunTimeHours/24 ;
  run;

  %if %dsn_obs(_TimeRecord) > 0 %then %do;
    ** Append single record to full signature file;
    ** PROC APPEND will create dataset on 1st use, then append record(s) on subsequent uses for same-named BASE= dataset;
    proc append force nowarn base=&OutSigName. data=_TimeRecord;
    run;
  %end;
  %else %do;
     %put;
     %put NOTE: &OutSigName was not created or updated due to upstream problems;
     %put;
  %end;

  ** Cleanup;
  proc datasets lib=work nowarn nolist nodetails;
    delete _TimeRecord;
  quit;
%MEND;


%** Macro validates parameters and aborts if invalid value is passed;
%MACRO ValidateParm(parm=, vals=, LogMessageStub=, mode=set_syscc);
  %** Check if valid or not;
  %if %index(&vals.,%upcase(&&&parm.))=0 %then %do;
    %put;
    %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
    %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
    %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
    %put ;
    %put %upcase(e)%upcase(rror:) Parm &parm. = &&&parm. and must be one of these values: &vals..;
    %if %lowcase(&mode) ne set_syscc %then %do;
      %put This program is now aborting;
      %put ;
      %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
      %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
      %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
      %abort cancel;
    %end;
    %else %do;
      %put Automatic macro variable SYSCC is now set to 99;
      %let syscc=99;
      %put ;
      %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
      %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
      %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
    %end;
  %end;
%MEND;

* End of qt_utility_macros.sas;